﻿using System.Collections.Generic;
using System.Windows;
using System.Windows.Media;

namespace Tonky_Utilities
{
    namespace Windows
    {
        partial class Lib_Tonky
        {
            /// <summary>
            /// Restituisce una collezione di oggetti enumerativi generici che va a cercare in depObj.
            /// Solitamente la si può usare per cercare dei tipi specifici di oggetti visuali su una Window WPF.
            /// </summary>
            /// <typeparam name="T">Tipo oggetto da cercare.</typeparam>
            /// <param name="depObj">Finestra Window WPF in cui cercare.</param>
            /// <returns>Sostituisce la collection Controls in WinForm.</returns>
            /// <remarks>
            /// http://stackoverflow.com/questions/974598/find-all-controls-in-wpf-window-by-type
            /// https://msdn.microsoft.com/it-it/library/system.windows.dependencyobject(v=vs.110).aspx
            /// https://msdn.microsoft.com/it-it/library/system.windows.dependencyobject(v=vs.110).aspx
            /// </remarks>
            public static IEnumerable<T> FindVisualChildren<T>(DependencyObject depObj) where T : DependencyObject
            {
                if (depObj != null)
                {
                    for (int i = 0; i < VisualTreeHelper.GetChildrenCount(depObj); i++)
                    {
                        DependencyObject child = VisualTreeHelper.GetChild(depObj, i);
                        if (child != null && child is T)
                        {
                            yield return (T)child;
                        }

                        foreach (T childOfChild in FindVisualChildren<T>(child))
                        {
                            yield return childOfChild;
                        }
                    }
                }
            }
        }
    }
}
