﻿using System;
using System.Linq;

namespace Gioco_15
{
    class clsGioco
    {
        private int[,] TabelloneGioco = new int[4, 4];

        private int[,] TabFine = new int[4, 4];

        /// <summary>
        /// Il costruttore di default inizializza opportunamente i valori del 
        /// tabellone di gioco sulle posizione di default!
        /// </summary>
        public clsGioco()
        {
            this.mosse = 0;
            int conta = 1;
            for(int i =0; i <= TabelloneGioco.GetUpperBound(0); i++)
            {
                for (int j =0; j <= TabelloneGioco.GetUpperBound(1); j++)
                {
                    if (conta <= 15)
                    {
                        TabelloneGioco[i, j] = conta;
                        conta++;
                    }
                    else
                        TabelloneGioco[i, j] = 0;
                }
            }

            // Memorizza la posizione finale !!!
            TabFine = (int[,])TabelloneGioco.Clone();
        }

        /// Proprietà di sola lettura che serve a contare il numero delle mosse effettuate.
        private int mosse = 0;
        public int ContaMosse
        {
            get { return mosse; }
        }

        /// <summary>
        /// Questa proprietà ritorna lo stato del tabellone di gioco.
        /// Può essere poi sfruttato dalla logica implementativa delle View.
        /// </summary>
        public int[,] StatusQuo()
        {
            // Tramite il clone della matrice spezza ogni riferimento allo status quo!
            // Lo faccio per preservare la logica MVC
            int[,] Buffer = new int[4, 4];
            Buffer = (int[,])TabelloneGioco.Clone();

            return Buffer;
        }


        #region "Definizione Eventi"

        /// <summary>
        /// Evento che si scatena non appena richiamo l'esecuzione di una nuova partita.
        /// </summary>
        public event EventHandler NewGame_Starting;
        protected virtual void OnNewGame_Starting(EventArgs e)
        {
            if (NewGame_Starting != null)
                NewGame_Starting(this, e);
        }

        /// <summary>
        /// Evento che viene rilasciato quando la nuova partita è già disponibile.
        /// </summary>
        public event EventHandler NewGame_Started;
        protected virtual void OnNewGame_Started(EventArgs e)
        {
            if (NewGame_Started != null)
                NewGame_Started(this, e);
        }

        /// <summary>
        /// Eventio che scaturisce se la mossa viene ritenuta valida.
        /// </summary>
        public event EventHandler MossaPossibile;
        protected virtual void OnMossaPossibile(EventArgs e)
        {
            if (MossaPossibile != null)
                MossaPossibile(this, e);
        }

        /// <summary>
        /// Eventio che scaturisce se la mossa viene ritenuta non valida.
        /// </summary>
        public event EventHandler MossaNonPossibile;
        protected virtual void OnMossaNonPossibile(EventArgs e)
        {
            if (MossaNonPossibile != null)
                MossaNonPossibile(this, e);
        }

        /// <summary>
        /// Evento che ha luogo quando la mossa è giunta la termine.
        /// </summary>
        public event EventHandler MossaTerminata;
        protected virtual void OnMossaTerminata(EventArgs e)
        {
            if (MossaTerminata != null)
                MossaTerminata(this, e);
        }

        public event EventHandler GameOver;
        protected virtual void OnGameOver(EventArgs e)
        {
            if (GameOver != null)
                GameOver(this, e);
        }
        #endregion


        #region "Metodi"

        /// <summary>
        /// Effettua la generazione di una nuova partita.
        /// </summary>
        public void NuovaPartita()
        {
            this.mosse = 0;
            OnNewGame_Starting(new EventArgs());

            //Random rnd = new Random(DateTime.Now.Hour +
            //                        DateTime.Now.Minute +
            //                        DateTime.Now.Second);
            Random rnd = new Random();

            // Sfrutta un semplice vettore per poi passare i dati sulla matrice
            int[] TabelloneSupporto = new int[16];

            int riga = 0;
            while (riga <= 14)
            {
                int tmp = rnd.Next(1, 16);
                bool trovato = false;

                for (int search = 0; search < TabelloneSupporto.Count(); search++)
                {
                    if (TabelloneSupporto[search] == tmp)
                    {
                        trovato = true;
                        break;
                    }
                }
                if (trovato == false)
                {
                    TabelloneSupporto[riga] = tmp;
                    riga++;
                }
            }

            for (int rig = 0; rig <= TabelloneGioco.GetUpperBound(0); rig++)
            {
                // Ciclo sulle righe

                for (int col = 0; col <= TabelloneGioco.GetUpperBound(1); col++)
                {
                    // Ciclo sulle colonne
                    // Copia il vettore di supporto nella matrice
                    int tmp = (rig * 4) + col;
                    TabelloneGioco[rig, col] = TabelloneSupporto[tmp];
                }
            }


            OnNewGame_Started(new EventArgs());
        }


        public enum TipoMossa { Su, Giu, Destra, Sinistra, NessunaMossa }

        /// <summary>
        /// Qui si determina se sia possibile muovere e poi viene eseguita la mossa.
        /// </summary>
        /// <param name="riga">Identifica il numero di riga che si desidera muovere.</param>
        /// <param name="colonna">Identifica il numero di colonna che si desidera muovere.</param>
        /// <returns>Restituisci un enumerativo per identificare da che parte ha mosso.</returns>
        public TipoMossa Mossa(int riga, int colonna)
        {
            // Indistintamente se la mossa è buona o meno la segna e fine!
            mosse++;

            int bufSWAP = -1;

            TipoMossa DirezioneMossa = TipoMossa.NessunaMossa;

            // Se la mossa è possibile determino anche la direzione da muovere
            if (((colonna - 1) >= 0) && (TabelloneGioco[riga, colonna - 1] == 0))
            {
                OnMossaPossibile(new EventArgs());
                DirezioneMossa = TipoMossa.Sinistra;

                bufSWAP = TabelloneGioco[riga, colonna];
                TabelloneGioco[riga, colonna] = TabelloneGioco[riga, colonna - 1];
                TabelloneGioco[riga, colonna - 1] = bufSWAP;
                OnMossaTerminata(new EventArgs());

                IsGameOver();
                return DirezioneMossa;
            }
            else if (((colonna + 1) <= TabelloneGioco.GetUpperBound(1)) && (TabelloneGioco[riga, colonna + 1]) == 0)
            {
                OnMossaPossibile(new EventArgs());
                DirezioneMossa = TipoMossa.Destra ;

                bufSWAP = TabelloneGioco[riga, colonna];
                TabelloneGioco[riga, colonna] = TabelloneGioco[riga, colonna + 1];
                TabelloneGioco[riga, colonna + 1] = bufSWAP;

                OnMossaTerminata(new EventArgs());

                IsGameOver();
                return DirezioneMossa;
            }
            else if (((riga - 1) >= 0) && (TabelloneGioco[riga - 1, colonna] == 0))
            {
                OnMossaPossibile(new EventArgs());
                DirezioneMossa = TipoMossa.Su;

                bufSWAP = TabelloneGioco[riga, colonna];
                TabelloneGioco[riga, colonna] = TabelloneGioco[riga - 1, colonna];
                TabelloneGioco[riga - 1, colonna] = bufSWAP;

                OnMossaTerminata(new EventArgs());

                IsGameOver();
                return DirezioneMossa;
            }
            else if (((riga + 1) <= TabelloneGioco.GetUpperBound(0)) && (TabelloneGioco[riga + 1, colonna] == 0))
            {
                OnMossaPossibile(new EventArgs());
                DirezioneMossa = TipoMossa.Giu;

                bufSWAP = TabelloneGioco[riga, colonna];
                TabelloneGioco[riga, colonna] = TabelloneGioco[riga + 1, colonna];
                TabelloneGioco[riga + 1, colonna] = bufSWAP;

                OnMossaTerminata(new EventArgs());

                IsGameOver();
                return DirezioneMossa;
            }

            OnMossaNonPossibile(new EventArgs());
            OnMossaTerminata(new EventArgs());

            // Qui prima di proseguire eseguo controllo se la partita è finita!?
            //IsGameOver();

            // Qui cado se e solo se nessuna mossa è possibile
            return DirezioneMossa;
        }


        /// <summary>
        /// Determina se il gioco è temrinato.
        /// </summary>
        /// <returns>
        /// Restituisce un booleano e poi di lancia un apposito evento.
        /// </returns>
        private bool IsGameOver()
        {
            bool result = true;

            for (int i=0; i <= TabelloneGioco.GetUpperBound(0); i++)
            {
                for(int j = 0; j <= TabelloneGioco.GetUpperBound(1); j++)
                {
                    if (TabelloneGioco[i, j] != TabFine[i, j])
                    {
                        j = TabelloneGioco.GetUpperBound(1) + 1;
                        i = TabelloneGioco.GetUpperBound(0) + 1;
                        result = false;
                    }
                }
            }

            //Qui scatena l'evento di fine giocata!
            if (result)
            {
                OnGameOver(new EventArgs());
                this.mosse = 0;
            }

            return result;
        }
        #endregion        
    }
}
